#include "texture2d.h"
#include <fstream>

Texture2D::Texture2D(char* filename) : Matrix<Color32>(COLUMN_MAJOR) {
	matrixFlag = true;
	using namespace std;
	ifstream fileBMP;
	BITMAPFILEHEADER bmfh;
	BITMAPINFOHEADER bmih;
	fileBMP.open(filename, ios::binary);
	fileBMP.read(reinterpret_cast<char*>(&bmfh), sizeof(bmfh));	
	fileBMP.read(reinterpret_cast<char*>(&bmih), sizeof(bmih));	
	numCols = bmih.biWidth;
	numRows = bmih.biHeight;
	length = numCols;
	ptr = new Color32[numRows * numCols];
	for(int y = 0; y < getHeight(); y++)
	for(int x = 0; x < getWidth(); x++) {
		fileBMP.read((char*) this->operator()(x, y).ptrComp(2), sizeof(char));
		fileBMP.read((char*) this->operator()(x, y).ptrComp(1), sizeof(char));
		fileBMP.read((char*) this->operator()(x, y).ptrComp(0), sizeof(char));
		this->operator()(x, y).refComp(3) = (GLubyte) 255;
	}
	fileBMP.close();
}

void Texture2D::setupTexture() {
	glGenTextures(1, &name);
	glBindTexture(GL_TEXTURE_2D, name);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, getWidth(), getHeight(), 0, GL_RGBA, GL_UNSIGNED_BYTE, ptrElem(0, 0));
}

void Texture2D::deleteTexture() {
	glDeleteTextures(1, &name);
}

void Texture2D::writeBMP(char* filename) {
	using namespace std;
	ofstream fileBMP;
	BITMAPFILEHEADER bmfh;
	BITMAPINFOHEADER bmih;
	int rowBytes = getWidth() * 3;
	int rowPadding = 0;
	while((rowBytes + rowPadding) % 4 != 0) rowPadding++;
	char* temp = new char;
	*temp = 0;
	bmfh.bfType = 19778;
	bmfh.bfSize = sizeof(bmfh) + sizeof(bmih) + (getWidth() * 3 + rowPadding) * getHeight();
	bmfh.bfReserved1 = 0;
	bmfh.bfReserved2 = 0;
	bmfh.bfOffBits = 54;
	bmih.biSize = sizeof(bmih);
	bmih.biWidth = getWidth();
	bmih.biHeight = getHeight();
	bmih.biPlanes = 1;
	bmih.biBitCount = 24;
	bmih.biCompression = 0;
	bmih.biSizeImage = getWidth() * getHeight() * 3;
	bmih.biXPelsPerMeter = 0;
	bmih.biYPelsPerMeter = 0;
	bmih.biClrUsed = 0;
	bmih.biClrImportant = 0;
	fileBMP.open(filename, ios::binary);
	fileBMP.write(reinterpret_cast<char*>(&bmfh), sizeof(bmfh));	
	fileBMP.write(reinterpret_cast<char*>(&bmih), sizeof(bmih));	
	for(int y = 0; y < getHeight(); y++) {
		for(int x = 0; x < getWidth(); x++) {
			fileBMP.write((char*) this->operator()(x, y).ptrComp(2), sizeof(char));
			fileBMP.write((char*) this->operator()(x, y).ptrComp(1), sizeof(char));
			fileBMP.write((char*) this->operator()(x, y).ptrComp(0), sizeof(char));
		}
		for(int i = 0; i < rowPadding; i++)	fileBMP.write(temp, sizeof(char));
	}
	fileBMP.close();
}